import os
import sys

from threading import *

def print_threads(writer=sys.stdout):
    row = "%-28s  %-36s  %-18s  %-8s  %-8s  %s"

    writer.write(row % ("Class", "Name", "Ident", "Alive", "Daemon", ""))
    writer.write(os.linesep)
    writer.write("-" * 120)
    writer.write(os.linesep)

    for thread in sorted(enumerate()):
        cls = thread.__class__.__name__
        name = thread.name
        ident = thread.ident
        alive = thread.is_alive()
        daemon = thread.daemon
        extra = ""
        #extra = thread._Thread__target

        writer.write(row % (cls, name, ident, alive, daemon, extra))
        writer.write(os.linesep)

class CuminDaemonThread(Thread):
    def __init__(self):
        super(CuminDaemonThread, self).__init__()

        self.name = self.__class__.__name__
        self.stop_requested = False

        self.setDaemon(True)
 
    def init(self):
        pass

    def stop(self, timeout=5):
        if self.stop_requested:
            return
        self.stop_requested = True
        if self.isAlive():
            self.join(timeout)

class CuminBlockingDaemonThread(CuminDaemonThread):
    def __init__(self):
        super(CuminBlockingDaemonThread, self).__init__()

        self._lock = Lock()
        self._condition = Condition(self._lock)

    def stop(self, timeout=5):
        try:
            self._condition.acquire()
            self.stop_requested = True
            self._condition.notify()
        finally:
            self._condition.release()

        if self.isAlive():
            self.join(timeout)
